/******************************************************************************
** 									     **
**    Zenobit 1dc0.2 : CITADEL Operating System				     **
**  ---------------------------------------------			     **
**									     **
**    Source: zcapp_avl_functions.h					     **
**   									     **
**    Purpose:								     **
**	Declares the various functions required by ZCA-AVL.		     **
**									     **
**    Author: DG.Ward.							     **
**									     **
**    Contact:								     **
**	comeherenightmare@hotmail.com					     **
**      warddg1@tekotago.ac.nz						     **
**	kyretzn@gmail.com						     **
**									     **
**    Date: 23th, April 2007						     **
**									     **
******************************************************************************/
#include "zcapp_avl_structures.h"

#ifndef _h_zcapp_avl_functions
#define _h_zcapp_avl_functions

	/* -------------------------------------------------------
 	  Yet to be fully ported to the Zenobit platform
		- Memory Management routines must be changed.
		  (ABMv2dcBr2, instead of Win32/UNIX Malloc).

		- Debugging routines must use ZSPRINTF instead of
		  LibC STDIO's PRINTF

		- Variants for Kernel and User space.
	
	  Port isn't urgent. [[ 230407 ~dgw ]]
	------------------------------------------------------- */
	#ifdef __ZENOBIT__
		#define __twk__zcapp_avl_kernelspace
		//#define __twk__zcapp_avl_userspace
		
		#ifdef __twk__zcapp_avl_kernelspace
			#include "../memory/kernelspace-abm/abm_functions.h"
		#else
			/* No port ... */
		#endif

		#define _ret_failure	__def__failure
		#define _ret_success	__def__success
	#else
		#include <stdio.h>
		#include <stdlib.h>

		#define _ret_failure	0
		#define _ret_success	1
	#endif

	#define _m_ins	0
	#define _m_rem	1

	// --- rotate_tree
	// *
	// *  purpose:
	// *	This function simply rotates left/right around the given
	// *	Subtree, returning the new Subtree root.
	// *
	// *  parameters:
	// *	subtree = Pointer to a valid AVL_NODE_T.
	// *	direction = _p_l = Rotate left.
	// *		    _p_r = Rotate right.
	// *
	// *  returns:
	// *	A pointer to the new Subtree root.
	// *
	// ---
	extern avl_node_t *rotate_tree(avl_node_t *subtree, unsigned char direction);

	// --- rebalance_tree
	// *
	// *  purpose:
	// *	It is the responsibility of this function to restore the given
	// *	Subtree to balance when it becomes either critical left or right.
	// *
	// *  parameters:
	// *	node = Pointer to a valid AVL_NODE_T, the unbalanced subtree.
	// *	mode = Mode of operation.
	// *	       _m_ins (Insertion rebalancing)
	// *	       _m_rem (Removal rebalancing)
	// *
	// *  returns:
	// *	Pointer to the Subtree's new root (As balanced as possible).
	// *
	// ---
	extern avl_node_t *rebalance_tree(avl_node_t *node, unsigned char mode);

	// --- create_node
	// *
	// *  purpose:
	// *	To create an initialized node with the requested key.
	// *
	// *  parameters:
	// *	key = desired node key.
	// *
	// *  returns:
 	// *	A pointer to your tasty, new and initialized AVL_NODE_T.
	// *	_ret_failure if the node could not be created.
	// *
	// ---
	extern avl_node_t *create_node(unsigned int key);

	// --- insert_node
	// *
	// *  purpose: 	
	// *	Allows you to insert whatever node into whatever desired tree.
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree pointer. (eg, &test_tree)
	// *	node = The initialized node to insert into tree.
	// *
	// *  returns:
	// *	A pointer to the node, on a successful insertion.
	// *	_ret_failure if the node is already present or if the tree or
	// *	node is null.
	// *
	// ---
	extern avl_node_t *insert_node(avl_node_t **tree, avl_node_t *node);

	// --- remove_node
	// *
	// *  purpose:
	// *	This function simply removes the desired node from whatever
	// *	tree. This function does free the node after removal.
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree pointer. (eg, &test_tree)
	// *	node = The node to remove.
	// *
	// *  returns:
	// * 	No return... 's clean, m'lord!
	// *
	// ---
	extern void remove_node(avl_node_t **tree, avl_node_t *node);

	// --- find_node
	// *
	// *  purpose:
	// *	This handy routine allows you to actually find nodes in your
	// *	glorious tree!
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree pointer. (eg. &test_tree)
	// *	key = Key to locate within the Tree.
	// *0
	// *  returns:
	// *	A pointer to the desired node, if it could be located.
	// *	Otherwise, _ret_failure.
	// *
	// ---
	extern avl_node_t *find_node(avl_node_t **tree, unsigned int key);

	// --- upbalance
	// *
	// *  purpose:
	// *	This function is specifically for updating / correcting the
	// *	tree's balances after a removal operation.
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree's pointer (eg. &test_tree)
	// *	node = The node to begin uptracing from.
	// *	stdir = the starting balance update.
	// *
	// *  returns:
	// *	No returns.
	// *
	// ---
	extern void upbalance(avl_node_t **tree, avl_node_t *node, unsigned char stdir);

#endif
