/******************************************************************************
** 									     **
**    Zenobit 1dc0.2 : CITADEL Operating System				     **
**  ---------------------------------------------			     **
**									     **
**    Source: zcapp_avl_insert_node.c					     **
**   									     **
**    Purpose:								     **
**	See explanation below.						     **
**									     **
**    Author: DG.Ward.							     **
**									     **
**    Contact:								     **
**	comeherenightmare@hotmail.com					     **
**      warddg1@tekotago.ac.nz						     **
**	kyretzn@gmail.com						     **
**									     **
**    Date: 23th, April 2007						     **
**									     **
******************************************************************************/
#include "zcapp_avl_functions.h"

	// --- insert_node
	// *
	// *  purpose: 	
	// *	Allows you to insert whatever node into whatever desired tree.
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree pointer. (eg, &test_tree)
	// *	node = The initialized node to insert into tree.
	// *
	// *  returns:
	// *	A pointer to the node, on a successful insertion.
	// *	_ret_failure if the node is already present or if the tree or
	// *	node is null.
	// *
	// ---
	avl_node_t *insert_node(avl_node_t **tree, avl_node_t *node) {
		if((!tree) || (!node)) return (avl_node_t*)_ret_failure;
		else if(!*tree) *tree = node;
		else {
			avl_node_t *iterator = *tree, *shadow = iterator;
			unsigned int path = 0;

			while(1) {
				path = (node->key > iterator->key);
				if(iterator->key == node->key) {
					free(node);
					return (avl_node_t*)_ret_failure;
				} else if(iterator->paths[path]) {
					shadow = iterator;
					iterator = iterator->paths[path];
				} else {
					iterator->paths[path] = node;
					node->paths[_p_u] = iterator;
					shadow = iterator;
					iterator = node;
					break;
				}
			}

			while(shadow) {
				path = (iterator->key > shadow->key);
				shadow->balance += (path ? 1 : -1);

				if(shadow->balance == _b_b) break;
				else if((shadow->balance == _b_lc) || (shadow->balance == _b_rc)) {
					if(!shadow->paths[_p_u]) *tree = rebalance_tree(shadow, _m_ins);
					else {
						path = (shadow->key > shadow->paths[_p_u]->key);
						shadow->paths[_p_u]->paths[path] = rebalance_tree(shadow, _m_ins);
					}
					break;
				}

				shadow = shadow->paths[_p_u];
				iterator = iterator->paths[_p_u];
			}
		}
		return node;
	}
