/******************************************************************************
** 									     **
**    Zenobit 1dc0.2 : CITADEL Operating System				     **
**  ---------------------------------------------			     **
**									     **
**    Source: zcapp_avl_rebalance_tree.c				     **
**   									     **
**    Purpose:								     **
**	See explanation below.						     **
**									     **
**    Author: DG.Ward.							     **
**									     **
**    Contact:								     **
**	comeherenightmare@hotmail.com					     **
**      warddg1@tekotago.ac.nz						     **
**	kyretzn@gmail.com						     **
**									     **
**    Date: 23th, April 2007						     **
**									     **
******************************************************************************/
#include "zcapp_avl_functions.h"

	// --- rebalance_tree
	// *
	// *  purpose:
	// *	It is the responsibility of this function to restore the given
	// *	Subtree to balance when it becomes either critical left or right.
	// *
	// *  parameters:
	// *	node = Pointer to a valid AVL_NODE_T, the unbalanced subtree.
	// *	mode = Mode of operation.
	// *	       _m_ins (Insertion rebalancing)
	// *	       _m_rem (Removal rebalancing)
	// *
	// *  returns:
	// *	Pointer to the Subtree's new root (As balanced as possible).
	// *
	// ---
	avl_node_t *rebalance_tree(avl_node_t *node, unsigned char mode) {
		avl_node_t *x_node = node->paths[(node->balance ? _p_r : _p_l)];
		avl_node_t *z_node = x_node->paths[(node->balance ? _p_l : _p_r)];

		unsigned char bal_buff = node->balance;

		if(x_node->balance == (bal_buff ? _b_lh : _b_rh)) {
			if(z_node->balance == (bal_buff ? _b_rh : _b_lh)) {
				x_node->balance = z_node->balance = _b_b;
				node->balance = (bal_buff ? _b_lh : _b_rh);
			} else if(z_node->balance == (bal_buff ? _b_lh : _b_rh)) {
				x_node->balance = (bal_buff ? _b_rh : _b_lh);
				node->balance = z_node->balance = _b_b;
			} else node->balance = x_node->balance = z_node->balance = _b_b;
			node->paths[(bal_buff ? _p_r : _p_l)] = rotate_tree(x_node, (bal_buff ? _p_r : _p_l));
		} else {
			if((mode == _m_rem) && (x_node->balance == _b_b)) {
				node->balance = (bal_buff ? _b_rh : _b_lh);
				x_node->balance = (bal_buff ? _b_lh : _b_rh);
			} else node->balance = x_node->balance = _b_b;
		}

		return rotate_tree(node, (bal_buff ? _p_l : _p_r));
	}
