/******************************************************************************
** 									     **
**    Zenobit 1dc0.2 : CITADEL Operating System				     **
**  ---------------------------------------------			     **
**									     **
**    Source: zcapp_avl_remove_node.c					     **
**   									     **
**    Purpose:								     **
**	See explanation below.						     **
**									     **
**    Author: DG.Ward.							     **
**									     **
**    Contact:								     **
**	comeherenightmare@hotmail.com					     **
**      warddg1@tekotago.ac.nz						     **
**	kyretzn@gmail.com						     **
**									     **
**    Date: 23th, April 2007						     **
**									     **
******************************************************************************/
#include "zcapp_avl_functions.h"

	// --- remove_node
	// *
	// *  purpose:
	// *	This function simply removes the desired node from whatever
	// *	tree. This function does free the node after removal.
	// *
	// *  parameters:
	// *	tree = Pointer to the Tree pointer. (eg, &test_tree)
	// *	node = The node to remove.
	// *
	// *  returns:
	// * 	No return... 's clean, m'lord!
	// *
	// ---
	void remove_node(avl_node_t **tree, avl_node_t *node) {
		if((!tree) || (!*tree) || (!node)) return;
		else {
			unsigned int path = node->key;
			avl_node_t *ancestor = node->paths[_p_u];
		
			if((node->paths[_p_l]) && (node->paths[_p_r])) {
				avl_node_t *replacement = node->paths[_p_l];
				while(replacement->paths[_p_r]) replacement = replacement->paths[_p_r];

				if(replacement->paths[_p_l]) {
					path = replacement->key;
					replacement->key = replacement->paths[_p_l]->key;
					free(replacement->paths[_p_l]);
					replacement->paths[_p_l] = 0;
					replacement->balance = _b_b;
					node->key = path;
					path = (replacement->key > replacement->paths[_p_u]->key);
				} else {
					path = (replacement->key > replacement->paths[_p_u]->key);
					replacement->paths[_p_u]->paths[path] = 0;
					node->key = replacement->key;
					free(replacement);
				}
				upbalance(tree, replacement->paths[_p_u], path);
				return;	
			} else if((!node->paths[_p_l]) && (!node->paths[_p_r])) {
				free(node);
				if(*tree == node) {
					*tree = 0;
					return;
				}
				path = (path > ancestor->key);
				ancestor->paths[path] = 0;
			} else {
				if(*tree == node) {
					*tree = node->paths[((node->balance-1) ? 1 : 0)]; 
					return;	
				} else {
					path = (path > ancestor->key);
					ancestor->paths[path] = node->paths[((node->balance-1) ? 1 : 0)];
					ancestor->paths[path]->paths[_p_u] = ancestor;
				}
				free(node);
			}
			upbalance(tree, ancestor, path);
		}
	}
